@@ -2,8 +2,9 @@ from geraldo.utils import get_attr_value, calculate_size
 from geraldo.widgets import Widget, Label, SystemField
 from geraldo.graphics import Graphic, RoundRect, Rect, Line, Circle, Arc,\
         Ellipse, Image
+from geraldo.base import GeraldoObject
 
-class ReportPage(object):
+class ReportPage(GeraldoObject):
     rect = None
     elements = None
     width = None
@@ -11,25 +12,28 @@ class ReportPage(object):
     def __init__(self):
         self.elements = []
 
-class ReportGenerator(object):
+    def get_children(self):
+        return self.elements
+
+class ReportGenerator(GeraldoObject):
     """A report generator is used to generate a report to a specific format."""
 
     _is_first_page = True
     _is_latest_page = True
     _current_top_position = 0
     _current_left_position = 0
-    _current_page_number = 0
+    _current_page_number = 1
     _current_object = None
     _current_queryset = None
     _generation_datetime = None
 
     # Groupping
     _groups_values = None
-    _groups_previous_values = None
+    _groups_working_values = None
     _groups_changed = None
     _groups_stack = None
 
-    # The rendered report have pages, each page is a ReportPage instance
+    # The rendered report has pages, each page is a ReportPage instance
     _rendered_pages = None
     _page_rect = None
 
@@ -40,10 +44,13 @@ class ReportGenerator(object):
         # Initializes some attributes
         self._rendered_pages = []
         self._groups_values = {}
-        self._groups_previous_values = {}
+        self._groups_working_values = {}
         self._groups_changed = {}
         self._groups_stack = []
 
+    def get_children(self):
+        return self._rendered_pages
+
     def execute(self):
         """This method must be overrided to execute the report generation."""
 
@@ -178,14 +185,13 @@ class ReportGenerator(object):
                     widget.para = self.make_paragraph(widget.text, self.make_paragraph_style(band, widget.style))
 
                     if widget.truncate_overflow:
-                        widget.keep = KeepInFrame(
+                        self.keep_in_frame(
+                                widget,
                                 self.calculate_size(widget.width),
                                 self.calculate_size(widget.height),
                                 [widget.para],
                                 mode='truncate',
                                 )
-                        widget.keep.canv = self.canvas
-                        widget.keep.wrap(self.calculate_size(widget.width), self.calculate_size(widget.height))
 
                         widget.left = band_rect['left'] + self.calculate_size(widget.left)
                         widget.top = self.calculate_top(temp_top, self.calculate_size(widget.top), self.calculate_size(widget.height))
@@ -384,7 +390,7 @@ class ReportGenerator(object):
     def render_bands(self):
         """Loops into the objects list to create the report pages until the end"""
         # Preparing local auxiliar variables
-        self._current_page_number = 0
+        self._current_page_number = self.report.first_page_number
         self._current_object_index = 0
         objects = self.report.get_objects_list()
 
@@ -404,7 +410,7 @@ class ReportGenerator(object):
             first_object_on_page = True
 
             # Generate the report begin band
-            if self._current_page_number == 0:
+            if self._current_page_number == 1:
                 self.render_begin()
 
             # Does generate objects if there is no details band
@@ -579,7 +585,7 @@ class ReportGenerator(object):
         changed = force_no_changed
 
         # Stores the previous group values
-        self._groups_previous_values = self._groups_values.copy()
+        self._groups_working_values = self._groups_values.copy()
 
         # Loops on groups until find the first changed, then all under it are considered
         # changed also
@@ -602,6 +608,9 @@ class ReportGenerator(object):
         """Renders the report headers using 'changed' definition calculated by
         'calc_changed_groups'"""
 
+        # Update working values for groups
+        self._groups_working_values = self._groups_values
+
         # Loops on groups to render changed ones
         for group in self.report.groups:
             if self._groups_changed.get(group, None) and\
@@ -614,21 +623,18 @@ class ReportGenerator(object):
         """Renders the report footers using previous 'changed' definition calculated by
         'calc_changed_groups'"""
 
-        reversed_groups = [group for group in self.report.groups]
-        reversed_groups.reverse()
-
         # Loops on groups to render changed ones
-        for group in reversed_groups:
+        for group in reversed(self.report.groups):
             if force or ( self._groups_changed.get(group, None) and\
                           self._groups_stack and\
                           self._groups_stack[-1] == group ):
-                #if not force and (not self._groups_stack or self._groups_stack[-1] != group):
-                #    continue
-                
                 if group.band_footer and group.band_footer.visible:
                     self.force_blank_page_by_height(self.calculate_size(group.band_footer.height))
                     self.render_band(group.band_footer)
 
+                if self._groups_stack:
+                    self._groups_working_values.pop(self._groups_stack[-1])
+
                 self._groups_stack.pop()
 
     def get_current_queryset(self):
@@ -650,17 +656,16 @@ class ReportGenerator(object):
         """Returns objects filtered in the current group or all if there is no
         group"""
 
-        filter = dict([(group.attribute_name, self._groups_previous_values.get(group, None))\
-                for group in self.report.groups if group in self._groups_stack])
+        filter_dict = dict([(group.attribute_name, value) for group, value in self._groups_working_values.items()])
 
         def filter_object(obj):
-            for k,v in filter.items():
+            for k,v in filter_dict.items():
                 if get_attr_value(obj, k) != v:
                     return False
 
             return obj
 
-        return [obj for obj in self.report.queryset if filter_object(obj)]
+        return filter(filter_object, self.report.queryset)
 
     # SubReports
 
@@ -723,3 +728,6 @@ class ReportGenerator(object):
         """Merge report default_style + band default_style + widget style"""
         raise Exception('Not implemented')
 
+    def keep_in_frame(self, widget, width, height, paragraphs, mode):
+        raise Exception('Not implemented')
+
